/*
 *    Library2.java
 *    Copyright (C) 2002 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3;

import org.greenstone.gsdl3.core.*;
import org.greenstone.gsdl3.util.*;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.File;
import java.io.IOException;
/**
 * A program to take XML strings from the command line and send 
 * them to the system.
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 * @version $Revision: 3292 $
 */
final public class Library2 {

    public static void main(String args[]) {

	if (args.length != 2) {
	    System.out.println("Usage: Library2 <gsdl3home> <sitehome>");
	    System.exit(1);
	}
	String gsdl_home = args[0]; 
	String transform_home = GSFile.xmlTransformDir(gsdl_home);
	String site_home = args[1];
	
	MessageRouter mr = new MessageRouter();
	mr.setSiteHome(site_home);
	mr.configure();
	
	// for doing xslt transforms
	XMLTransformer transformer = new XMLTransformer();

	BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
	String query=null;
	String request=null;
	String result=null;
	String collect=null;
	String service=null;
	String info=null;
	String content=null;
	while (true) {
	    System.out.println("Please enter a  query type (choose from: describe, query) or 'exit' to quit");
	    try {
		query = br.readLine();
		query.trim();
		
		if (query.startsWith("exit")) {
		    System.exit(1);
		}
		if (query.equals("describe")|| query.equals("query")) {
		    System.out.println("\nPlease enter a collection name (or <enter> for no collection)");
		    collect = br.readLine();
		    collect.trim();
		    System.out.println("\nPlease enter a service name (or <enter> for no service)");
		    service = br.readLine();
		    service.trim();
		    if (query.equals("describe")) {
			
			System.out.println("\nPlease enter an info name (or <enter> for no info)");
			info = br.readLine();
			info.trim();
			
		    } else if (query.equals("query") ) {
			System.out.println("\nPlease enter the content to your query (in xml, all on one line - sorry about that)");
			content=br.readLine();
			content.trim();
		    }
		    
		    request = "<message><request type='"+query+"' to='";
		    
		    if (!collect.equals ("")) {
			request += collect+"/";
		    }
		    if (!service.equals("")) {
			request += service+"/";
		    }
		    request += "'";
		    if (query.equals("describe") && !info.equals("")) {
			request += " info='"+info+"'/>";
		    }
		    else if (query.equals("query") && !content.equals("")) {
			request +=">"+content+"</request>";
		    } else {
			request += "/>";
		    }


		    request += "</message>";
		    
		    System.out.println("Sending message to MessageRouter:\n"+request);
		    result = mr.process(request);
		    
		    System.out.println("Raw Result:\n"+result+"\n");
		    
		    // transform the result
		    
		    // select a stylesheet
		    String stylesheet = null;
		    if (query.equals("describe")) {
			if (collect.equals("")&&service.equals("")) {
			    stylesheet = transform_home+File.separatorChar+"home.xsl";
			} else if (!collect.equals("")) {
			    stylesheet = transform_home+File.separatorChar+"collect_about.xsl";
			} else if (!service.equals("")) {
			    stylesheet = transform_home+File.separatorChar+"service_about.xsl";
			}
		    } else if (query.equals("query")) {
			stylesheet = transform_home+File.separatorChar+"query_result.xsl";
		    }
		    if (stylesheet != null) {
			File style = new File(stylesheet);
			if (!style.exists()) {
			    System.err.println("stylesheet doesn't exist");
			} else {
			
			    String transformed_result = transformer.transform(stylesheet, result);
			    System.out.println("Transformed result:\n"+transformed_result);
			}
		    } else {
			System.out.println("no appropriate stylesheet found");
		    }
		} else {
		    System.out.println("your request was not understood, sorry");
		}
		
	    }  catch (IOException e) {
		System.err.println("Library2 IO exception:"+e.getMessage());
	    }
	    
	} // while (true
    }
    
}

    
