package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element;

import java.util.HashMap;
public class PageAction extends Action {


    public String process (String xml_in) {
	
	Element message = converter_.getDOM(xml_in).getDocumentElement();
	Element request = (Element)message.getElementsByTagName("request").item(0);

	String info = request.getAttribute("info");
	// remove the page/ bit
	info = GSPath.removeFirstLink(info);
	String page_name = GSPath.getFirstLink(info);
	if (page_name.equals("")) {
	    // for now send an error. change to always display the home page?
	    return GSHTML.errorPage(" page action specified, but no page specified");
	}
	if (page_name.equals("home")) {
	    return homePage(request);
	} else if (page_name.equals("about")) {
	    return aboutPage(request);
	} else {
	    return GSHTML.errorPage("page: "+page_name+" not yet implemented");
	}
    }

    protected String homePage(Element request) {
	
	// first, get the list of collections from mr
	
	Element coll_list_message = doc_.createElement("message");
	Element coll_list_request = doc_.createElement("request");
	coll_list_message.appendChild(coll_list_request);
	coll_list_request.setAttribute("type", "describe");
	coll_list_request.setAttribute("info", "collectionList");
	
	Element coll_list_response = (Element)mr_.process(coll_list_message);
	if (coll_list_response==null) {
	    return GSHTML.errorPage("couldn't query the mr about collections");
	}
	
	System.out.println("the initial xml response is");
	System.out.println(converter_.getString(coll_list_response));

	NodeList colls = coll_list_response.getElementsByTagName("collection");
	
	Element metadata_message = doc_.createElement("message");
	Element metadata_request = doc_.createElement("request");
	metadata_message.appendChild(metadata_request);
	metadata_request.setAttribute("type", "describe");	
	for (int i=0; i<colls.getLength(); i++) {
	    // get the metadata for each one
	    Element c = (Element)colls.item(i);
	    String name = c.getAttribute("name");
	    
	    metadata_request.setAttribute("to", name); // overwrites the old value
	    Element metadata_response = (Element)mr_.process(metadata_message);
	    Element coll = (Element)metadata_response.getElementsByTagName("collection").item(0);
	    GSXML.mergeMetadataLists(c, coll); // add the metadata to the original response
	}

	//return GSHTML.htmlSafe(converter_.getString(coll_list_response));
	//now the full response is in coll_list_response
	//String stylesheet = GSFile.stylesheetPath(gsdl_home_, site_home_,
	//					  display_home_, "home.xsl");
	String stylesheet = GSFile.stylesheetPath(config_, "home.xsl");

	Element system = doc_.createElement("system");
	//Element sitename = doc_.createElement("sitename");
	system.setAttribute("sitename", "localsite");
	coll_list_response.appendChild(coll_list_response.getOwnerDocument().importNode(system, true));
	
	System.out.println("the second xml response is");
	System.out.println(converter_.getString(coll_list_response));

	String transformed_result = transformer_.transform(stylesheet,
							   coll_list_response);    
	return transformed_result;
    	     
    } // homePage

    protected String aboutPage(Element request) {

	// get the params out
	HashMap params = GSXML.extractParams((Element)GSXML.getChildByTagName(request, "paramList"));
	String coll_name = (String)params.get("c");
	String lang = (String)params.get("l");
	if (lang==null || lang.equals("")) {
	    lang="en"; // the default
	}
	System.out.println("lang="+lang);
	Element coll_about_message = doc_.createElement("message");
	//coll_about_message.setAttribute("xml:lang", lang);
	Element coll_about_request = doc_.createElement("request");
	coll_about_message.appendChild(coll_about_request);
	coll_about_request.setAttribute("type", "describe");
	coll_about_request.setAttribute("to", coll_name);
	Element coll_about_response = (Element)mr_.process(coll_about_message);
	coll_about_response.setAttribute("lang", lang);
	//return GSHTML.htmlSafe(converter_.getString(coll_about_response));
	//String stylesheet = GSFile.stylesheetPath(gsdl_home_, site_home_,
	//     				  display_home_, "about.xsl");
	String stylesheet = GSFile.stylesheetPath(config_, "about.xsl");

	System.out.println("stylesheet="+stylesheet);
	String transformed_result = transformer_.transform(stylesheet,
							   coll_about_response);    
	return transformed_result;
	
	
    }
}
