package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.Text; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

import java.util.HashMap;

public class QueryAction extends Action {
    

    public String process (String xml_in) {

	Element message = converter_.getDOM(xml_in).getDocumentElement();
	Element request = (Element)message.getElementsByTagName("request").item(0);
	
	String info = request.getAttribute("info");
	// remove the q/ bit
	info = GSPath.removeFirstLink(info);
	String query_type = GSPath.getFirstLink(info);
	
	if (query_type.equals("")) {
	    query_type = "text";
	}

	if (query_type.equals("text")) {
	    return textQuery(request);
	}
	if (query_type.equals("field")) {
	    return fieldQuery(request);
	}
	return unknownQuery(request);

	
    }
    
    protected String textQuery(Element request) {

	Element query_request = doc_.createElement("message");
	query_request.appendChild(doc_.importNode(request, true));

	String stylesheet = GSFile.stylesheetPath(config_, "textquery.xsl");
	
	//String stylesheet = GSFile.stylesheetPath(gsdl_home_, site_home_,
	//     				  display_home_, "textquery.xsl");
	if (stylesheet==null) {
	    return GSHTML.errorPage("textquery stylesheet not found!");
	}
	
	HashMap params = GSXML.extractParams((Element)GSXML.getChildByTagName(request, "paramList"));
	String query = (String)params.get("q");
	if (query==null|| query.equals("")) {
	    // if there is no q set, just output the search box
	    String transformed_result = transformer_.transform(stylesheet,
							       query_request);    
	    return transformed_result;
	}
	
	// else do the request, output search box and results
	Element mr_message = doc_.createElement("message");
	Element mr_request = doc_.createElement("request");
	mr_message.appendChild(mr_request);

	mr_request.setAttribute("type", "query");
	String to = (String)params.get("c"); // collection name
	to = GSPath.appendLink(to, "TextQuery");
	mr_request.setAttribute("to", to);
	
	// ignore paramList for now
	Element paramList = doc_.createElement("paramList");
	mr_request.appendChild(paramList);

	// content - contains the query string
	Element content = doc_.createElement("content");
	Text text = doc_.createTextNode(query);
	content.appendChild(text);
	mr_request.appendChild(content);
	//return GSHTML.errorPage(GSHTML.htmlSafe(converter_.getString(mr_message)));

	Element result1 = (Element)mr_.process(mr_message);
	// this result is the list of docs.
	// now take that list, and get the Titles
	// for now, just create a whole new request

	Element resource_list = (Element)result1.getElementsByTagName("resourceList").item(0); // should always be a resource list - but it may be empty

	Element first_resource = (Element)GSXML.getChildByTagName(resource_list, "resource");
	if (first_resource == null) { // no docs found - dont bother getting metadata
	    Element result_response = (Element)GSXML.getChildByTagName(result1, "response");
	    
	    query_request.appendChild(doc_.importNode(result_response, true));
	    
	    String transformed_result = transformer_.transform(stylesheet,
							       query_request);
	    
	    return transformed_result;
	}
	// get the Title metadata
	Element mr_message2 = doc_.createElement("message");
	Element mr_request2 = doc_.createElement("request");
	mr_message2.appendChild(mr_request2);

	mr_request2.setAttribute("type", "query");
	to = (String)params.get("c"); // collection name
	to = GSPath.appendLink(to, "MetadataRetrieve");
	mr_request2.setAttribute("to", to);
	Element content2 = doc_.createElement("content");
	mr_request2.appendChild(content2);

	// the first part of the content is the doc list
	content2.appendChild(doc_.importNode(resource_list, true));

	// the second part of the content is the metadata list
	Element metadata_list = doc_.createElement("metadataList");
	Element title = doc_.createElement("element");
	title.setAttribute("name", "Title");
	metadata_list.appendChild(title);
	content2.appendChild(metadata_list);

	//return GSHTML.errorPage(GSHTML.htmlSafe(converter_.getString(mr_request2)));
	Element result2 = (Element)mr_.process(mr_message2);
	
	Element result2_response = (Element)GSXML.getChildByTagName(result2, "response");
	
	query_request.appendChild(doc_.importNode(result2_response, true));
	//return GSHTML.errorPage(GSHTML.htmlSafe(converter_.getString(query_request)));

	String transformed_result = transformer_.transform(stylesheet,
          				   query_request);
    
    return transformed_result;
	
    }

    protected String fieldQuery(Element request) {
	return GSHTML.errorPage("field query not implemented");


    }
    protected String unknownQuery(Element request) {

	return GSHTML.errorPage("unknown query subtype - unknown query not implemented");
    }
}
