package org.greenstone.gsdl3.core;

import org.greenstone.gsdl3.util.*;
import org.greenstone.gsdl3.action.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

// other java classes
import java.io.File;
import java.util.HashMap;

public class Receptionist implements ModuleInterface {

    /** the set up variables */
    protected ConfigVars config_=null;

    /** container Document to create XML Nodes */
    protected Document doc_=null; 
    /** a converter class to parse XML and create Docs */
    protected XMLConverter converter_=null;

    /** the message router that it talks to */
    protected ModuleInterface mr_=null;

    /** the list of actions */
    protected HashMap action_map_=null;

    public Receptionist() {
	converter_ = new XMLConverter();
	doc_ = converter_.newDOM();
	action_map_= new HashMap();
    }
 
    /** the set up vars must be set before configure called*/
    public void setConfigVars(ConfigVars config) {
	config_ = config;
    }
    /** sets the message router  - it should already be created and 
     * configured before  being passed to the receptionist*/
    public void setMessageRouter(ModuleInterface m) {
       mr_ = m;
    }

    public boolean configure() {

	if (config_==null) {
	    System.err.println("You must set the config variables before calling configure");
	    return false;
	}
	if (mr_==null) {	   
	    System.err.println("You must set  the mr  before calling configure");
	    return false;
	}

	// for now, just statically add in the actions
	// actions may disappear if xslt is all that is needed
	// or they should be dynamically loaded
	Action a = new PageAction();
	a.setConfigVars(config_);
	a.setMessageRouter(mr_);
	a.configure();
	action_map_.put("p", a);

	a = new QueryAction();
	a.setConfigVars(config_);
	a.setMessageRouter(mr_);
	a.configure();
	action_map_.put("q", a);

	a = new BrowseAction();
	a.setConfigVars(config_);
	a.setMessageRouter(mr_);
	a.configure();
	action_map_.put("b", a);

	a = new ResourceAction();
	a.setConfigVars(config_);
	a.setMessageRouter(mr_);
	a.configure();
	action_map_.put("r", a);
	return true;
    }

    public String process(String xml_in) {

	Element message = converter_.getDOM(xml_in).getDocumentElement();
	NodeList requests = message.getElementsByTagName("request");
	if (requests.getLength()==0) {
	    return GSHTML.errorPage("no requests in the message");
	}

	//  should only be one request
	Element req = (Element)requests.item(0);
	String type = req.getAttribute("type"); // returns "" if no att of this name
	if (!type.equals("action")) {
	    return GSHTML.errorPage("wrong type in request - should be action");
	}
	String info = req.getAttribute("info");
	String action = GSPath.getFirstLink(info);
	if (action.equals("")) {
	    return GSHTML.errorPage(" no action specified");
	}
	// pass to appropriate action
	
	ModuleInterface mi = (ModuleInterface)action_map_.get(action);
	if (mi==null) {
	    return GSHTML.errorPage("no action for "+action);
	}
	// just pass a string for now - better to pass the doc cos already
	// parsed it, but we want a string back
	String page = mi.process(xml_in);
	
	// output the page
	return page;
    }

    public Node process(Node xml_in) {
	return xml_in; // for now

    }


}
